c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine srlyrf(time,ndate,nxf,nyf,nzf,nfspec,ctlonf,ctlatf,
     &                  cloudf,temprf,tfsurf,uf,vf,presf,
     &                  htf,flandf,lstrsf,lsnowf,ch2of,rainf,
     &                  psfcf,depvelf,ifine)
c
c-----ATDM v1.00
c
c     Performs the following tasks for a fine grid:
c     (1) estimates PBL parameters such as the monin-obukhov
c         length and ustar
c     (2) estimates dry deposition velocities at each grid cell
c         and for each species (gas and particle)
c         depvelf - landuse-weighted deposition velocities (m/s)
c
c-----95/03/08:
c     -from UAM-V Revision 1.2  94/02/10  21:00:39  tcm
c     -modifications to streamline combined gas/particulate deposition
c      calculations
c-----95/10/17:
c     -revised calculation of cloud-reduced solar insolation
c-----95/11/03:
c     -revised rho in call to partdep to be in correct g/m3 units
c-----95/11/08:
c     -added code to skip cells covered by a fine grid
c-----95/12/01:
c     -changed rs and rb dimensions from mxspec to mxgas to agree with
c      SURLYR.F and SURFCE.F
c     -Good bye
c-----96/09/13:dfg
c     -added call to micsurf instead of surfce when micro CB4 
c      mechanism is specified.
c     -added include 'ctlflg.cmd' for mechanism flag used in this routine.
c
c     Input arguments:
c               time:           model time
c               ndate:          model date
c               nxf:            number of fine grid cells in x-direction
c               nyf:            number of fine grid cells in y-direction
c               nfspec:         number of modeled species
c               ctlonf:         array of grid cell longitudes (deg)
c               ctlatf:         array of grid cell latitudes (deg)
c               cloudf:         array of cloud coverage
c               temprf:         array of temperature (K)
c               tfsurf:         array of surface temperature (K)
c               uf:             array of u-component winds (km/min)
c               vf:             array of v-component winds (km/min)
c               presf:          array of layer pressure (mb)
c               htf:            array of layer interface heights (m)
c               flandf:         array of fractional land use values
c               lstrsf:         array of stress codes
c               lsnowf:         array of snow codes
c               ch2of:          array of water vapor concentration (kg/kg)
c               rain:           array of rainfall rates (in/hr)
c               psfcf:          array of surface pressure (mb)
c               ifine:          fine grid index
c     Output arguments:
c               depvelf:        fine grid deposition velocities (m/s)
c
c     Routines called:
c               solar     pblmod    aerody    bndry     surfce    partdep
c               jtocal
c     Called by:
c               atdm
c
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'parfin.cmd'
      include 'deffin.cmd'
      include 'aerosl.cmd'
      include 'latlon.cmd'
      include 'comchem.cmd'
      include 'com3.cmd'
      include 'ctlflg.cmd'
      include 'advec.cmd'
c
      dimension ixff0(maxfin), ixff1(maxfin),
     &          jyff0(maxfin), jyff1(maxfin)
      dimension ctlonf(nxf,nyf), ctlatf(nxf,nyf),
     &          cloudf(nxf,nyf,nzf), temprf(nxf,nyf), tfsurf(nxf,nyf), 
     &          uf(nxf,nyf), vf(nxf,nyf), presf(nxf,nyf),
     &          htf(nxf,nyf), flandf(nxf,nyf,mlus), lstrsf(nxf,nyf), 
     &          lsnowf(nxf,nyf), ch2of(nxf,nyf), 
     &          rainf(nxf,nyf),
     &          psfcf(nxf,nyf), depvelf(nxf,nyf,nfspec)
c
      dimension ra(mlus), rs(mxgas,mlus), rb(mxgas,mlus)
      real zz0(mlus), solflx(10), cldadj(11), cosz(10), wsrf(mlus)
      dimension flndcl(mlus), vmeanp(mxspec,mlus)
      integer iprt jprt
      real lv
      logical lprt
c
      data zz0 /3., 0.25, 0.05, 1., 1., 1., 0.0001, 0.002, 
     &          0.15, 0.1, 0.1 /
      data solflx /915., 890., 840., 750., 630., 500., 345., 
     &          190., 60., 0./
      data cldadj /1., 0.91, 0.84, 0.79, 0.75, 0.72, 0.68, 
     &             0.62, 0.53, 0.41, 0.23/
      data cosz /1., 0.9962, 0.9659, 0.9063, 0.8192, 0.7071, 0.5736, 
     &           0.4226, 0.2588, 0.0872/
      data eps /0.622/, rd /287./, e0 /6.11/, lv /2.5e6/, rv /461./
      data iprt /41/, jprt/21/
c
c----Set up some parameters relating the grids
c
      do 5 if1 = 1,numfin
        if (ifgptr(if1) .ne. ifine) go to 5
        ixff0(if1) = (ixfb(if1) - ixfb(ifine))*nhf(ifine) + 2
        ixff1(if1) = (ixfe(if1) - ixfb(ifine) + 1)*nhf(ifine) + 1
        jyff0(if1) = (jyfb(if1) - jyfb(ifine))*nhf(ifine) + 2
        jyff1(if1) = (jyfe(if1) - jyfb(ifine) + 1)*nhf(ifine) + 1
        if (ixff0(if1).le.2 .or. ixff1(if1).ge.nxf .or.
     &      jyff0(if1).le.2 .or. jyff1(if1).ge.nyf) then
          write(*,*) ' erroneous grid parameters in srlyrf'
          write(*,*) ' ixff0, ixff1, jyff0, jyff1: ',ixff0,ixff1,
     &                 jyff0, jyff1
          write(*,*) 'if1, ifine: ',if1,ifine
          stop
        endif
   5  continue
c
c-----Initialize some contants and arrays that are needed
c
      do 133 lu = 1,mlus
        ra(lu) = 0.
        do 131 l = 1,ngas
          rb(l,lu) = 0.0
          rs(l,lu) = 0.0
  131   continue
  133 continue
      do 136 l = 1,nfspec
        do 135 j = 1,nyf
          do 134 i = 1,nxf
            depvelf(i,j,l) = 0.
  134     continue
  135   continue
  136 continue
c
c-----Convert date to day,month and year
c
      if (ndate .lt. 100000) then
        call jtocal(ndate,icdate)
        ny = icdate/10000
        nm = icdate/100-ny*100
        nd = icdate-nm*100-ny*10000
      else
        ny = ndate/10000
        nm = ndate/100-ny*100
        nd = ndate-nm*100-ny*10000
      endif
      ny = ny+1900
c     write(*,*)'SRLYRF: year,mo,day,time',ny,nm,nd,time
c
c-----Calculate the surface deposition rate row by row
c
      do 100 j = 2,nyf-1
c
c-----Loop over each column element
c
        do 90 i = 2,nxf-1
c
c-----Skip cells covered by another finer grid
c
          do 10 if1 = 1,numfin
            if (ifgptr(if1).eq.ifine) then
              if (i.ge.ixff0(if1) .and. i.le.ixff1(if1)
     &            .and. j.ge.jyff0(if1) .and. j.le.jyff1(if1))
     &            go to 90
            endif
   10     continue
          lprt = .false.
c          if (ifine .eq. 1 .and. i.eq.iprt .and. j.eq.jprt) lprt = .true.
c
c-----Calculate solar zenith angle and solar flux
c
          rllng = -ctlonf(i,j) 
          rllt = ctlatf(i,j)
          call solar(rllt,rllng,tz,ny,nm,nd,time,zcoef)
          if (lprt) write(*,*)'lat,lon,tz,zcoef',
     &                         rllt,rllng,tz,time,zcoef

          do 95 ncz = 2,10
            if (zcoef.ge.cosz(ncz).and.zcoef.lt.cosz(ncz-1)) then
              icz = ncz-1
              goto 96
            endif
   95     continue
          icz = 10

   96     continue 
          cldmax = 0.
          do 97 k = 1,nzf
            cldmax = amax1(cldmax,cloudf(i,j,k))
            if (lprt) write(*,*) 'cloudf ',k,cloudf(i,j,k)
   97     continue
          ica = nint(10.*cldmax + 0.5)
          gsun = cldadj(ica)*solflx(icz)
          if (lprt) then
            write(*,*)'icz,cldmax,ica,cldadj,solflx,gsun'
            write(*,*) icz,cldmax,ica,cldadj(ica),solflx(icz),gsun
          endif
c
c-----Load environmental parameters; convert winds from km/min to m/s
c
          tcell = temprf(i,j)
          pcell = presf(i,j)
          tscell = tfsurf(i,j)
          pscell = psfcf(i,j)
          topcell = htf(i,j)
          qcell = ch2of(i,j)
          if (lstagr) then
            ubar = (uf(i,j) + uf(i-1,j))/2.
            vbar = (vf(i,j) + vf(i,j-1))/2.
            wcell = 16.67*sqrt(ubar**2 + vbar**2)
            wcell = amax1(0.01,wcell)
          else
            wcell = 16.67*sqrt(uf(i,j)**2 + vf(i,j)**2)
            wcell = amax1(0.01,wcell)
          endif
c
          tv = tcell*(eps + qcell)/(eps*(1. + qcell))
          rhoair = 1.e5*pcell/(rd*tv)
          ev = qcell*pcell/(qcell + eps)
          es = e0*exp((lv/rv)*(1./273. - 1./tcell))
          rhum = 100.*ev/es
          if (lprt) then
            write(*,*)'t,p,ts,ps,top,q,w,rh,rho'
            write(*,*)tcell,pcell,tscell,pscell,topcell,qcell,
     &                wcell,rhum,rhoair
          endif
c         
c-----Loop over non-zero land use categories for each cell to get
c     gaseous ra & rb, and particle deposition velocity
c
          do 50 lu = 1,mlus
            flndcl(lu) = flandf(i,j,lu)
            if (flandf(i,j,lu).lt.1.0e-3) goto 50
            z00 = zz0(lu)
            if (lprt) write(*,*)'doing LU # ',lu,flandf(i,j,lu),z00
c
c-----Estimate the monin-obukhov length and ustar; save over-water ustar
c     for later use
c
            call pblmod(wcell,tcell,tscell,pcell,pscell,topcell,
     &                 z00,elinv,ustar,wref)
            wsrf(lu) = wref
            if (lu.eq.7) then
              ustrw = ustar
              z00 = 1.6e-6/ustrw + 1.6e-3*ustrw**2
            endif
            if (lprt) write(*,*)'elinv,ustar,wref,z00 ',
     &                           elinv,ustar,wref,z00
c
c-----Estimate the aerodynamic resistance, and neutral 
c     ustar for later use
c
            call aerody(ustar,elinv,z00,ra(lu))
            ustrn = sqrt(wsrf(lu)/ra(lu))
            if (lprt) write(*,*)'ra,ustrn',ra(lu),ustrn
c
c-----Estimate the laminar flow resistance for gases
c
            call bndry(ustrn,rb(1,lu))
c
c-----Calcualte the deposition velocity of particles
c
            do 40 l = ngas+1,nfspec
               if (l.eq.knh4n .or. l.eq.knh4s) goto 40
               isd = iszdist(l)
               rho = 1.e6*rhop(l)
               call partdep(rsize,rmass(1,isd),nbin,ustrn,
     &                      z00,rho,tcell,ra(lu),rhoair,
     &                      vmeanp(l,lu))
               if (lprt) then
                 write(*,*)'rho,isd,vmeanp ',l,rho,isd,vmeanp(l,lu)
               endif
   40       continue
   50     continue
c
c-----Estimate the canopy (surface) resistance for gases
c
c*** add call to micsurf when mechanism is specified. (dfg)
c
        if (lmech) then
          call micsurf(nm,gsun,ustrw,wsrf,tcell,rhum,lstrsf(i,j),
     &            lsnowf(i,j),flndcl,rainf(i,j),rs,pcell)
        else
          call surfce(nm,gsun,ustrw,wsrf,tcell,rhum,lstrsf(i,j),
     &                lsnowf(i,j),flndcl,rainf(i,j),rs)
        endif
          if (lprt) write(*,*)'lstrsf,lsnowf,rain ',
     &                         lstrsf(i,j),lsnowf(i,j),rainf(i,j)
c
c-----Calculate the deposition velocity (m/s)
c     running sum over landuse becomes landuse-weighted deposition velocity
c     for species k
c
          do 85 k = 1,ngas
             do 55 lu = 1,mlus
                if (flandf(i,j,lu).lt.1.e-3) goto 55
                vdep = 1.0/(ra(lu) + rb(k,lu) + rs(k,lu))
                if (lprt) write(*,*)'k,lu,ra,rb,rs,vdep ',
     &                               k,lu,ra(lu),rb(k,lu),rs(k,lu),vdep
                depvelf(i,j,k) = depvelf(i,j,k) - flandf(i,j,lu)*vdep
  55         continue
             if (lprt) write(*,*)'depvel ',k,depvelf(i,j,k)
  85      continue
          do 86 k = ngas+1,nfspec
             if (k.eq.knh4n .or. k.eq.knh4s) goto 86
             do 56 lu = 1,mlus
                if (flandf(i,j,lu).lt.1.e-3) goto 56
                vdep = vmeanp(k,lu)
                depvelf(i,j,k) = depvelf(i,j,k) - flandf(i,j,lu)*vdep
  56         continue
             if (lprt) write(*,*)'depvel ',k,depvelf(i,j,k)
  86      continue
          if (knh4n.ne.0) depvelf(i,j,knh4n) = depvelf(i,j,kpno3)
c
   90   continue
  100 continue
c
      return
      end
